function [m, derr, mRec, merr] = ConjGrad_L_Lt(L,Lt,d,m,Niter, mtru)
% [m, derr, mRec, merr] = ConjGrad_L_Lt(L,Lt,d,m,Niter, mtru)
% conjugate gradient method for solving
%         d = L mtru
%         m is initial guess, Niter is # of iteration
% L, Lt   linear operators, implemented as anonymous function handles
% m, derr, mRec, merr  if starting from 0, have Niter+1 'elements', 
%         otherwise, Niter;  each is like || x - x0 ||
% To calculate merr, mtru is required!
% see also ConjGrad_MtrxInv.m

nm    = numel(m);
INIT0 = max(abs(m(:))) < 1e-12;
if INIT0
  if nargout>2,  mRec=zeros([nm, Niter+1]); end
  if nargout>1,  derr=zeros(1, Niter+1);    end
  if nargout>3,  merr=zeros(1, Niter+1);    end
  kshf = 1;
  %--
  if nargout>1,  derr(1) = sqrt( d(:)'*d(:) ); end
  if nargout>3,  merr(1) = sqrt( mtru(:)' * mtru(:) ); end
  r = Lt(d);
else
  if nargout>2,  mRec=zeros([nm, Niter]); end
  if nargout>1,  derr=zeros(1, Niter);    end
  if nargout>3,  merr=zeros(1, Niter+1);  end
  kshf = 0;
  %--
  r  = -Lt(L(m) - d);
end
p  = r;
ro = r(:)' * r(:);
tic
for k = 1:Niter
  Ap = Lt(L(p));
  alp= ro/(p(:)'*Ap(:));
	m  = m + alp * p;
	if nargout > 1
    d1 = L(m) - d;
		derr(k+kshf) = sqrt( d1(:)'*d1(:) );
	  if nargout > 2
	    mRec(:, k+kshf) = m(:);
      if nargout > 3
        er = m - mtru;
        merr(k+kshf)  = sqrt( er(:)' * er(:) );
      end
    end
  end
  if (k==Niter),  break,  end
	r  = r - alp * Ap;
	rn = r(:)'*r(:);
	p  = r + (rn/ro)*p;
	ro = rn;
	%--
	tlaps = toc;
  disp(['k=',int2str(k),' Telapsed=',num2str(tlaps/60),' Tremain=',...
    num2str(tlaps/k*(Niter-k)/60),' (min)'])
  if nargout>1, disp(['Data err norm=', num2str(derr(k+kshf))]), end
end
return
%===========

L = randn(10, 8);
m0= randn(8, 1);  m = zeros(8,1);
d = L*m0;

f = @(x) (aLinearOperator(x, L, 1));
ft= @(x) (aLinearOperator(x, L, 0));

[m, derr, mRec, merr] = ConjGrad_L_Lt(f,ft,d,zeros(size(m)),8, m0);
figure
plot(derr/derr(1),'.-'), hold on
plot(merr/merr(1),'ro-')
m - m0  % right!
