% LSMF_script.m

if matlabpool('size') < 1,  matlabpool,   end

nz  = 80;    nx = 150;     dx = 15;       nzx = nz*nx;   
x12=[1,nx]*dx; z12=[1,nz]*dx;  % for plotting range
vp  = 2e3;   vs = 1.15e3;
reflz = round(0.8*nz/(vp/vs)); % depth of reflector

lams= 10*dx;  f0 = vs/lams; dt = 1/f0/12;
ric= ricker_wavelet(f0, dt)';
Nt = numel(ric) + round( 2* sqrt(nz^2 +(nx/2)^2) *dx/vs/dt );

ns = 12;                   % shots
sx = linspace(1,150,ns);   sz = -1;
gx = 1:nx;   gz = 0;       % geophones

% make traveltime tables
TTPs = travTimesTable_cnstV(vp,nz,nx,dx,sx, sz);
TTPg = travTimesTable_cnstV(vp,nz,nx,dx,gx, gz);
TTSs = travTimesTable_cnstV(vs,nz,nx,dx,sx, sz);
TTSg = travTimesTable_cnstV(vs,nz,nx,dx,gx, gz);

Rtru = zeros(nz,nx);
Rtru(reflz,:) = 0.5;
% one reflec. model, both P & S wave reflected

% Observed data:
dtru = kirch_cnsV_ForwMd_w_TT(Rtru(:), vp, ric, dt, Nt, TTPg, TTPs) + ...
       kirch_cnsV_ForwMd_w_TT(Rtru(:), vs, ric, dt, Nt, TTSg, TTSs);
       
Iter=60;
%% CG:    define L and Lt operators, which CG expects.
L = @(x)(kirch_cnsV_ForwMd_w_TT(x, vp, ric, dt, Nt, TTPg, TTPs) + ...
         kirch_cnsV_ForwMd_w_TT(x, vs, ric, dt, Nt, TTSg, TTSs) );
Lt= @(x)(kirch_cnsV_mig_w_TT(   x, vp, ric, dt,     TTPg, TTPs) + ...
         kirch_cnsV_mig_w_TT(   x, vs, ric, dt,     TTSg, TTSs) );
[m, derr, mRec, merr] = ConjGrad_L_Lt(L,Lt,dtru,zeros(nzx,1),Iter, Rtru(:));

dp = kirch_cnsV_ForwMd_w_TT(m, vp, ric, dt, Nt, TTPg, TTPs);
ds = kirch_cnsV_ForwMd_w_TT(m, vs, ric, dt, Nt, TTSg, TTSs);

figure
subplot(2,2,1)
plot(derr/derr(1),'.-'),  hold on
plot(merr/merr(1),'ro-'), title('LSMF errors by CG')
axis([1, Iter+1, 0.1, 1])
xlabel('iteration number')
ylabel('relative error')
legend('data error', 'model error')

subplot(2,2,3), 
imagesc(x12, z12, reshape(m, [nz,nx])), title('LSMF image by CG')
xlabel('X (m)')
ylabel('Z (m)')

subplot(3,2,2), 
imagesc(x12, [1,Nt]*dt, dtru(:,:,floor(ns/3))), title('observed d')
set(gca,'ydir','normal'),  ylabel('t (s)')

subplot(3,2,4), 
imagesc(x12, [1,Nt]*dt, dp(:,:,floor(ns/3))), title('predicted d_p')
set(gca,'ydir','normal'),  ylabel('t (s)')

subplot(3,2,6), 
imagesc(x12, [1,Nt]*dt, ds(:,:,floor(ns/3))), title('predicted d_s')
set(gca,'ydir','normal'), xlabel('X (m)'), ylabel('t (s)')

print -dpng LSMF0_CG.png

