function trc = kirch_cnsV_ForwMd_w_TT(R, v, ric, dt, Nt, TTg, TTs)
% prestack Kirch forward modeling w/ const v w/ Trav Tim Table
% v                1 x 1        vel
% TTg [TTs]  (nz*nx) x ng [ns], travel time (s)
% R          (nz*nx) x 1        reflectivity
% ric           Lric x 1        source wavelet
%        column vectors!!!!!
%----
% trc    Nt x ng x ns     recorded ns CSGs

[nzx,ng] = size(TTg);
ns       = size(TTs, 2);
Lric     = numel(ric);
Mt1 = Nt-Lric+1;   % such that the final resuting length is Nt
MxR = max(abs(R));
Rthr= 1e-5*MxR;
v2  = v*v;
trc  = zeros(Nt, ng, ns);
parfor is= 1:ns
  dat= zeros(Mt1, ng);  % NOTE: 'dat' inside parfor is treated as private
  for im= 1: nzx
    if (abs(R(im)) < Rthr), continue, end
    t1  = TTs(im, is);
    for ig= 1:ng
      t2  = TTg(im, ig);
      t   = 1 + round((t1+t2)/dt);
      if t>Mt1, continue, end
      dat(t,ig) = dat(t,ig) + R(im)/(t1*t2*v2);  % use 2d geometr. spreading will be better
    end
  end
  trc(:,:,is)= conv2(dat(:,:), ric);
end
return
%===============
% test
cd 'E:\yunsong\Documents\Courses\KAUST\Seismic_Imaging\Kirch_cnstV'

if matlabpool('size') < 1,  matlabpool,   end

nz = 60;   nx = 120;  dx = 10;  v = 2000;
lam= 10*dx;
f0 = v/lam;
dt = 1/f0/10;
ric= ricker_wavelet(f0, dt);  ric = ric(:);
Nt = numel(ric) + round(2*sqrt((nz+1)^2 + nx^2)*dx/v/dt);
figure, plot(ric,'.-'),  pause(0.5)

ns = 6;
sx = linspace(nx/(2*ns), nx/ns, ns);
gx = 1:nx;
TTs = travTimesTable_cnstV(v,nz,nx,dx,sx, -1);
TTg = travTimesTable_cnstV(v,nz,nx,dx,gx, 0);

%--- dot-prod test:
R1 = randn(nz,nx);  R1=R1(:);  R1(abs(R1)<1e-5*max(abs(R1))) = 0;
R2 = randn(nz,nx);  R2=R2(:);  R2(abs(R2)<1e-5*max(abs(R2))) = 0;

trc1 = kirch_cnsV_ForwMd_w_TT(R1, v, ric, dt,  Nt, TTg, TTs);
trc2 = kirch_cnsV_ForwMd_w_TT(R2, v, ric, dt,  Nt, TTg, TTs);
m1   = kirch_cnsV_mig_w_TT(trc1,  v, ric, dt,      TTg, TTs);
dtp1 = R2(:)'*m1(:);
dtp2 = trc1(:)'*trc2(:);
dtp2 / dtp1

figure,
for k = 1: min(numel(sx), 4)
  subplot(2,2,k), imagesc(trc1(:,:,k))
end
figure
imagesc(reshape(m1,[nz,nx]))

