function mig = kirch_cnsV_mig_w_TT(trc, v, ric, dt, TTg, TTs)
% prestack Kirch migration w/ const v
% trc Nt x ng x ns   CSG's, codistributed along ns
% v    1 x 1         vel
% ric         COLUMN VECTOR
% TTg, TTs   (nz*nx) x ng [ns], travel time (s)
%----
% mig nz*nx X 1   migration image, composite. all labs have identical copy

nzx  = size(TTg, 1);
[Nt, ng, ns] = size(trc);
Lric = numel(ric);
Mt1  = Nt - Lric + 1;
mig  = zeros(nzx, ns);
v2   = v*v;
parfor is = 1:ns
  tmp = xcorr2(trc(:,:,is), ric);  % important, like RTM, correlating source
  % wavefield and receiver field.  Computational efficiency, in Kirchoff,
  % we can do correlation first, and then smearing.
  tmp = tmp(Lric: Nt,:);  % of length Nt
  for im = 1: nzx
    t1  = TTs(im, is);
    for ig = 1:ng
      t2 = TTg(im, ig);
      t  = 1 + round((t1+t2)/dt);
      if t>Mt1, continue, end
      mig(im,is)= mig(im,is) + tmp(t,ig)/(t1*t2*v2);   % use 2d geometr. spreading will be better
    end
  end
end
mig = sum(mig,2);
return
%=================

