function [ric, t] = ricker_wavelet(f0, dt, L)
% 0-phase ricker wavelet in time domain, unit-norm
% L  max possible length of ric

if nargin < 3,  L = 1e4;  end
np = min(ceil(1./(f0*dt)), floor((L-1)/2));  % avoid ceil @ integer
t  = (-np : np) * dt;
t2 = (t *(pi*f0)).^2;
env= exp(-t2);
ric= (1 - 2*t2).*env;
N  = 2*np+1;
den= 2*(N/12)^2;
ric= ric.*(1-exp(-(1:N).^2 ./den)).*(1-exp(-(-N:-1).^2 ./den));
smr= sum(ric);
sme= sum(env);
ric= ric - smr/sme * env;  % to make it 0-mean

ric= ric/norm(ric);
return
%==============
f0  = 10;    dt = 2e-3;
ric = ricker_wavelet(f0, dt);
figure
plot(ric,'.-'), hold on
ric(1)/max(abs(ric))

R = fft(ric, 1024);
figure,  plot(abs(R))

hr = hilbert(ric);
env= abs(hr);
plot(env,'r')   % envelope indeed :-)

